# Moritz Orbach 2005, 2006, 2007, 2008, 2010, 2011, 2021
# gnu@apfelboymchen.net
# http://apfelboymchen.net/gnu/configstuff/scripts/
#
# shell behaviour on interactive use

# noclobber (don't overwrite files with '>'. Force with '>|'
set -C

#
## shopts
#

# globbing
shopt -s extglob  # extended globbing in pathname expansion, e.g. !(bla)
shopt -s dotglob  # include dotfiles in pathname expansion
GLOBIGNORE=".."   # but don'tmatch ".." in "rm -r .*"

# "no match" instead of the wildcard. This can lead to problems
# https://bugzilla.redhat.com/show_bug.cgi?id=622655
# http://wiki.debian.org/Teams/BashCompletion/Proposals/Roadmap
# but returning the glob if it didn't match is just strange.
#shopt -s failglob

shopt -s globstar # "**" expansion

# misc
shopt -s no_empty_cmd_completion # no completion on an empty line
shopt -s lastpipe                # save a process
shopt -s cdable_vars             # 'cd a' is 'cd $a' if $a is set. The file ~/.vars is sourced for this purpose.
shopt -s execfail                # don't exit the shell if "exec something" failed
shopt -s huponexit               # send HUP to all child processes on exit
shopt -s direxpand               # expand $dir to /real/dir
shopt -u sourcepath              # script to source in $PATH? How? Why?

#
## history
#
shopt -s histappend # append to history instead of overwriting it on logout
shopt -s lithist    # CRs in history
shopt -s histreedit # keep the line after a failed history subsitution ('!')

# specifies history expansion character, quick substitution character
# to their default and: leaves the history comment character empty!
histchars='!^'

# don't save repeated commands multiple times (ignoredups)
# don't save lines starting with space. Useful to not save sensitive commands! (ignorespace)
# ignoreboth: both
HISTCONTROL="ignoreboth"

#
## Keys
#

# readline-config, a.k.a. MOTHER OF GOD, STOP THE BEEPING!
INPUTRC="$BASEDIR/inputrc"
bind -f "$INPUTRC"

# edit ~/vars and create some variables with cd'able directories for quick 
# access to frequently used directories on the machine
test -f "$HOME/.vars" && . ~/.vars

# put ~/bin into PATH if not already set
if [ -d "$HOME/bin" ]; then
  tmppath="${PATH#$HOME/bin}"
  test "$tmppath" == "$PATH" && PATH="$HOME/bin:$PATH"
  unset tmppath
fi


# vim:ft=sh
# birth: Do 16. Sep 19:21:45 CEST 2021 aus ../bashrc
